#### Packages required
library(ade4)
library(sf)
library(sp)
library(rgdal)
library(rgeos)
library(raster)
library(maptools)
library(dismo)
library(ecospat)
library(geosphere)
library(countrycode)
library(hypervolume)
library(outliers)
library(class)
library(ggplot2)
library(dplyr)
library(tidyr)
library(scales)
library(reshape2)
library(ggpubr)
library(grDevices)
library(extrafont)



#####################################################################
path <- "D:/PHD_projects/Symbrenthia/2_period_20220206/Alian_invasive_niche_shift"
r <- raster::getData("worldclim", var = "bio", res = 2.5, path = path)
r <- r[[c(1, 3, 4, 5, 6, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19)]]
#####################################################################
lista_variables <- list.files(pattern='*.asc',full.names=TRUE)
r <- stack(lista_variables)

#####################################################################
"overlap.eq.gen" <- function(repi, z1, z2) {
  sim.o.D <- NaN
  while (is.nan(sim.o.D)) {
    
    # overlap on one axis
    ##z1$sp is the environmental values for occurences of the species 
    occ.pool <- c(z1$sp, z2$sp)  # pool of random occurrences
    occ.pool <- occ.pool[sample(length(occ.pool))]
    row.names(occ.pool)<-c() # remove rownames 
    rand.row <- base::sample(1:length(occ.pool), length(z1$sp)) # random reallocation of occurrences to dataset
    sp1.sim <- occ.pool[rand.row]
    sp2.sim <- occ.pool[-rand.row]
    
    z1.sim <- ecospat.grid.clim.dyn(z1$glob, z1$glob1, data.frame(sp1.sim), R = length(z1$x), th.sp=0, th.env=0)  # gridding in this function there is something
    z2.sim <- ecospat.grid.clim.dyn(z2$glob, z2$glob1, data.frame(sp2.sim), R = length(z2$x), th.sp=0, th.env=0)
    
    o.i <- ecospat.niche.overlap(z1.sim, z2.sim, cor = F)  # overlap between random and observed niches FAlSE because all environments
    sim.o.D <- o.i$D  # storage of overlaps
    sim.o.I <- o.i$I
    
  }
  return(c(sim.o.D, sim.o.I))
}


"niche.equivilency" <- function (z1, z2, rep, alternative = "greater", ncores = 1) 
{
  R <- length(z1$x) #100
  l <- list()
  obs.o <- ecospat.niche.overlap(z1, z2, cor = FALSE) 	#observed niche overlap, FAlSE because all environments
  if (ncores == 1) {
    sim.o <- as.data.frame(matrix(unlist(lapply(1:rep, overlap.eq.gen, z1=z1, z2=z2)), byrow = TRUE, ncol = 2))
  }
  
  colnames(sim.o) <- c("D", "I")
  l$sim <- sim.o
  l$obs <- obs.o
  if (alternative == "greater") {
    l$p.D <- (sum(sim.o$D >= obs.o$D) + 1)/(length(sim.o$D) +  1)
    l$p.I <- (sum(sim.o$I >= obs.o$I) + 1)/(length(sim.o$I) + 1)
  }
  if (alternative == "lower") {
    l$p.D <- (sum(sim.o$D <= obs.o$D))/(length(sim.o$D)) ###why is it +1 
    
    l$p.I <- (sum(sim.o$I <= obs.o$I) + 1)/(length(sim.o$I) + 1)
  }
  return(l)
}

### For mapping densities and overlap
col_E <- rgb(1,0,0,0.2) #non-native colours
col_N <- rgb(0,0,1,0.2) #native colours

"map_overlap" <- function(y, status){
  y_E <- y[status == "E"]
  y_N <- y[status == "N"]
  dens_E <- density(y_E) 
  dens_N <- density(y_N)
  xlim <- range(dens_E$x, dens_N$x) #x axis range
  ylim <- range(0, dens_E$y, dens_N$y)
  ylim[2] <- ylim[2] + 0.1
  hist(y_E, proba = T,  xlim = xlim, ylim = ylim, col= 'white', density = 10, angle = 135, main = "2013-2021", xlab = "Between-Class PCA 1", ylab = "Density", cex.lab=1.5, cex.axis=1.5)
  hist(y_N, proba = T, add = T, col = 'white', density = 10, angle = 45)
  polygon(dens_E, density = -1, col = col_E)
  polygon(dens_N, density = -1, col = col_N)
  mat <- cbind(dens_E$x, dens_E$y)
  mat <- rbind(mat, mat[1,])
  pol_E <- st_polygon(list(mat))
  mat <- cbind(dens_N$x, dens_N$y)
  mat <- rbind(mat, mat[1,])
  pol_N <- st_polygon(list(mat))
  pol_inter <- st_intersection(st_buffer(pol_E,0),  st_buffer(pol_N,0))
  plot(pol_inter, add = T, col = "grey", ylim=c(0,1))
  rug(y_N, side = 1, line = -0.15, col = col_N, tck = 0.03, lwd=2)
  rug(y_E, side = 1, line = -0.15, col = col_E, tck = 0.03, lwd=2)
}
##################################################################

windows()
niche_expansion_1 <- NULL
sample_size_1 <- NULL 
gridnumbers <- NULL
par(mfrow=c(1,1))
par(mar=c(5, 5, 5, 5))

####################################################################



species <- read.csv("2013_2021.csv", header = T)
head(species)
#remove duplicate
dups <- duplicated(species[, c("lon", "lat")])
sum(dups) # number of duplicates
species <- species[!dups, ] #remove duplicates


## native data
Native <- species[which(species$status=='N'),]
rownames(Native) <- 1:nrow(Native)
lat <- Native$lat 
lon <- Native$lon
coords <- data.frame(x=lon,y=lat) #creating a data frame of the latitudes and longitudes 
N_origional <- nrow(coords)
#--> function below uses  nndist to calculae distance between two points
coords <- ecospat.occ.desaggregation(xy=coords, min.dist=0.02, by=NULL) 
N_thinned <- nrow(coords)
index <- match(rownames(coords), rownames(Native))
points <- SpatialPointsDataFrame(coords, data=Native[index,], proj4string = r@crs) #this converts the lat and longs into spatial data
values <- raster::extract(r,points, cellnumbers=T) #extracts the bioclim data for the coodrinate points we have 
## See how many points ecist within the same grid cell 
grid_N <- as.data.frame(table(values[,1]))
grid_N <- nrow( grid_N[grid_N$Var1[ grid_N$Freq > 1],])

status <- points$status
Native_df <- cbind.data.frame(coordinates(points), status, values)  #binds the bioclim data 
Native_df <- Native_df[!duplicated(Native_df$cells),]
Native_df <- Native_df[,-4]


## non-native data
Exotic <- species[which(species$status=='E'),]
rownames(Exotic) <- 1:nrow(Exotic)
lat <- Exotic$lat
lon <- Exotic$lon
coords <- data.frame(x=lon,y=lat)
E_origional <- nrow(coords)
coords<-ecospat.occ.desaggregation(xy=coords, min.dist=0.02, by=NULL) 
E_thinned <- nrow(coords)
index <- match(rownames(coords), rownames(Exotic))
points <- SpatialPointsDataFrame(coords, data=Exotic[index,], proj4string = r@crs) #this converts the lat and longs into spatial data
values <- raster::extract(r,points, cellnumbers=T)
## See how many points ecist within the same grid cell 
grid_E <- as.data.frame(table(values[,1]))
grid_E <- nrow(grid_E[ grid_E$Var1[ grid_E$Freq > 1],])
spec_grid <- cbind(grid_N, grid_E, species)
colnames(spec_grid) <- c('Grid_N', 'Grid_E', "Species")
gridnumbers <- rbind(gridnumbers, spec_grid)

status <- points$status
Exotic_df <- cbind.data.frame(coordinates(points), status, values)  #binds the bioclim data 
Exotic_df <-  Exotic_df[!duplicated(Exotic_df$cells),]
Exotic_df <-  Exotic_df[,-4]


## save number of occurences
sizes <- cbind( N_origional, N_thinned , E_origional, E_thinned )
#rownames(sizes) <- c(species82[3])
rownames(sizes) <- "Symbrenthia"
sample_size_1 <- rbind( sample_size_1, sizes)

## buid raw dataset for species
#first remove all NAs
Exotic_df <- Exotic_df[complete.cases(Exotic_df), ] 
Native_df <- Native_df[complete.cases(Native_df), ]
#dataset for the whole envirnoment
Native_exotic_df <- rbind(Exotic_df, Native_df)
status <- as.factor(Native_exotic_df$status)

### PCA and between class analysis
## perform a principal component analysis of a data frame 
pca.env <- dudi.pca(Native_exotic_df[,4:20], scannf = F, nf = 10) ##so can first make a pca of all, look at eigen values ect. 

## perform between-class analysis : highlihgts the differences between groups 
status <- as.factor(status)
bet1 <- bca(pca.env, status, scan = FALSE, nf = 9) 
#bet1$ratio #between-class ratio

## explore correlaitons to each bio variable to axis
correlation_to_pca.env <- as.data.frame (bet1$as)#see the correlation with the normal pca for the axis. 
correlation_to_pca <- t(correlation_to_pca.env) #this is a data frame of the contribution of each origional pca axis to the between-pca axis
rownames(correlation_to_pca) <-  "Symbrenthia"
correlation_to_bio_variables <- as.data.frame (bet1$co) #to see the contribution of each worldclim BIO variable
correlation_to_bio_variables <- t(correlation_to_bio_variables) 
rownames(correlation_to_bio_variables) <-  "Symbrenthia"
#ecospat.plot.contrib(bet1$co)

## save interesting values  
between_class <- as.data.frame(bet1$ls) #data frame of the first axis of between class analysis
data <-  cbind(status, between_class) #binding the two together
#the scores for the whole environment
scores.globclim <- data$CS1
#scores for the presences 
scores.sp.ex <- data$CS1[which(data$status=='E')]
scores.sp.nat <-data$CS1[which(data$status=='N')] 
#scores for the whole study region, same as presences as no considering whole study area 
scores.clim.ex <- data$CS1[which(data$status=='E')]
scores.clim.nat <-data$CS1[which(data$status=='N')] 

### Evaluate niche shifts
## Grid the whole environment 
#correct species occupancy and whole enviornment into densities and can take climatic avaibility into account 
grid.clim.nat <- ecospat.grid.clim.dyn(glob=scores.globclim, glob1= scores.sp.nat, sp=scores.sp.nat, R=100, th.sp=0)
#glob= all background dataframe, glob1= of the native/invasive envinoment range of the species (for each one), envirnomental variable for occurances only, 
grid.clim.inv <- ecospat.grid.clim.dyn(glob=scores.globclim, glob1=scores.sp.ex , sp=scores.sp.ex, R=100, th.sp=0)

## Overlap
# D overlap
D.overlap <- ecospat.niche.overlap (grid.clim.nat, grid.clim.inv, cor=F)$D #niche overlap. 

# Niche expansion : delimiting niche categories and quantifying niche dynamics in analogue climates
niche.dyn.whole <- ecospat.niche.dyn.index (grid.clim.nat, grid.clim.inv, intersection = NA) #if intersection= NA means no intersection of analogous envirnoment
niche.dyn.whole$dynamic.index.w 

## Equivilency Test
eq.test <- NA
eq.test <- niche.equivilency(grid.clim.nat, grid.clim.inv, rep=1000, alternative = "lower", ncores = 1)

## save results
row <- cbind.data.frame(D.overlap, eq.test$p.D, niche.dyn.whole$dynamic.index.w[[1]],niche.dyn.whole$dynamic.index.w[[2]], niche.dyn.whole$dynamic.index.w[[3]])
names(row) <- c('D.overlap', 'eq.test', 'expansion', 'stability', 'Unfilling')
rownames(row) <- "Symbrenthia"
row <- cbind ( row,  correlation_to_pca,  correlation_to_bio_variables) #add the correlations of bio and pca axis to each row 
niche_expansion_1 <- rbind(niche_expansion_1, row)
bet1$ls[,1]


## plot results
map_overlap(bet1$ls[,1], status)


########data frame
niche_expansion_1
sample_size_1

niche_results <- niche_expansion_1[,1:5]
sample_size <- sample_size_1
niche_results
sample_size
