#### Raster manipulation & Ecological niche modeling ####
## Modified by: Min-Wei Chai

library(sp)
library(raster)

#### Raster data manipulation ----------------------------------------------------

##stack the layer
lista_variables <-list.files(pattern='*.tif',full.names=TRUE)
worldclim <- stack(lista_variables)


#Assign Taiwan coordinates
e<-extent(119.8,122,21.5,25.4)
e<-extent(113,124.5,21.5,26.5)
e<-extent(98.5,124.5,14,30.5)

ecrop <- crop(worldclim,e)
writeRaster(stack(ecrop[[c(1:92)]]), names(ecrop[[c(1:92)]]),
            bylayer=TRUE, format='ascii', overwrite=T)


##### input your own sampling site ----------------------------------------------
sites <- read.csv("past_2012_GLM_distribution.csv", header = T)
head(sites)

max(sites$lon)
min(sites$lon)
max(sites$lat)
min(sites$lat)

## Plot on map
# Option 1: use world map
library(maptools)
data(wrld_simpl)
plot(wrld_simpl, xlim=c(101.3,122), 
     ylim=c(14.3,26.5), axes=TRUE, 
     col="grey")
#for lunica data
plot(wrld_simpl, xlim=c(98.5,124.5), 
     ylim=c(14,31), axes=TRUE, 
     col="grey")
box()
formosanus <- subset(sites,species_name=="formosanus")
lunica <- subset(sites,species_name=="lunica")
points(formosanus$lon, formosanus$lat, col="orange", pch=20, cex=0.75)
points(lunica$lon, lunica$lat, col="green", pch=20, cex=0.75)
points(sites$lon, sites$lat, col="red", pch=20, cex=0.75)

# Option 2: use Taiwan raster file
taiwan_alt <- raster("alt.asc")
windows()
plot(taiwan_alt)
points(formosanus$lon, formosanus$lat, col="blue", pch=20, cex=0.75)
points(lunica$lon, lunica$lat, col="red", pch=20, cex=0.75)
points(sites$lon, sites$lat, col="blue", pch=20, cex=0.75)

#remove duplicate
dups <- duplicated(sites[, c("lon", "lat")])
sum(dups) # number of duplicates
sites <- sites[!dups, ] #remove duplicates

## Get value from point ---------------------------------------------
head(sites)
#[row,column]
sitemap <- sites[,2:3]
lista_variables <- list.files(pattern='*.asc',full.names=TRUE)
a <- stack(lista_variables)
#colnames(site1)=c("lon", "lat")
coordinates(sitemap) <- c("lon", "lat")  # set spatial coordinates
sp<-SpatialPoints(sitemap)
b<-extract(a,sp,method="bilinear")

c<-cbind(sites,b)
write.csv(c,file="past_2012_wt.csv", row.names = F)

#### Environmetal variables filtering by VIF -----------------------
library(usdm)
env <- read.csv("past_2012_wt.csv", header = T)
head(env)

# Climate
vifstep(env[4:22],th=10)
env_1 <- env[c("bio_12","bio_18","bio_2","bio_4","bio_8","bio_9")]

vifcor(env_1[1:6],th=0.6)
## Run ENM in MAXENT --------------------------------------------------------

## Read results from MAXENT -------------------------------------------------
output <- raster("lunica_avg.asc")
library(RColorBrewer)
color<-colorRampPalette(c("gray","green","yellow","orange","red"))

jpeg("lunica_past_2012.jpg", h=8, w=7,units = 'in',res = 300)
plot(output,col=color(100),zlim=c(0,1))

dev.off()
